//! Core type definitions for graph algorithms.

/// Vertex identifier (node ID)
pub type VertexId = i32;

/// Edge index in adjacency structure
pub type EdgeIndex = i64;

/// Edge weight type
pub type Weight = f64;
